import React, { Component } from "react";

import Form from "../components/form";
import HOCcomponent from "../components/HOC/resize";
import Button from "../components/button/button";

const list = [
	{
		id: 1,
		editable: true,
		firstName: "Tony",
		lastName: "Montana",
		age: 50,
		hobby: ["drugs", "guns"]
	}
];

class Main extends Component {
	state = {
		list: list,
		editableForm: false,
		item: {
			editable: false,
			firstName: "",
			lastName: "",
			age: "",
			hobby: [
				{
					id: Math.random()
						.toString()
						.substr(2, 100),
					value: ""
				}
			]
		}
	};

	addHobby = () => {
		this.setState(prevState => ({
			...prevState,
			item: {
				...prevState.item,
				hobby: prevState.item.hobby.concat({
					id: Math.random()
						.toString()
						.substr(2, 100),
					value: ""
				})
			}
		}));
	};

	rmHobby = id => {
		this.setState(prevState => ({
			...prevState,
			item: {
				...prevState.item,
				hobby: prevState.item.hobby.filter(el => el.id !== id)
			}
		}));
	};

	onChange = e => {
		const { name, value, checked, type, id } = e.target;

		if (name === "hobby") {
			console.log("id", id);
			console.log("ww", this.state.item);
			return this.setState(prevState => ({
				...prevState,
				item: {
					...prevState.item,
					hobby: prevState.item.hobby.map(el => (el.id === +id ? { ...el, value } : el))
				}
			}));
		}

		return this.setState(prevState => ({
			...prevState,
			item: {
				...prevState.item,
				[name]: type === "checkbox" ? checked : value
			}
		}));
	};

	onSubmit = () => {
		const { editableForm } = this.state;

		if (editableForm) {
			this.setState(prevState => ({
				...prevState,
				list: prevState.list.map(el =>
					el.id === prevState.item.id
						? { ...prevState.item, hobby: prevState.item.hobby.map(el => el.value) }
						: el
				)
			}));

			return this.resetFormHandler();
		}

		this.setState(prevState => ({
			...prevState,
			list: prevState.list.concat({
				...prevState.item,
				id: Math.random()
					.toString()
					.substr(2, 100),
				hobby: prevState.item.hobby.map(el => el.value)
			})
		}));

		return this.resetFormHandler();
	};

	resetFormHandler = () =>
		this.setState({
			item: {
				editable: false,
				firstName: "",
				lastName: "",
				age: "",
				hobby: [
					{
						id: Math.random()
							.toString()
							.substr(2, 100),
						value: ""
					}
				]
			},
			editableForm: false
		});

	removeHandler = id =>
		this.setState(prevState => ({
			...prevState,
			list: prevState.list.filter(el => el.id !== id)
		}));

	editHandler = id => {
		const item = this.state.list.find(el => el.id === id);
		this.setState({ editableForm: true, item });
	};

	render() {
		const { list } = this.state;
		return (
			<>
				<div style={{ border: "1px solid #eee", textAlign: "center", padding: 10 }}>
					Now your screen size:
					<p>W = {this.props.w}</p>
					<p>H = {this.props.h}</p>
				</div>
				<div className="box">
					<div className="list">
						{list.map(el => (
							<div className="list__item" key={el.id}>
								<Button
									text="x"
									className="list__rm-btn"
									// onClick={() => this.removeHandler(el.id)} />
									onClick={this.removeHandler.bind(null, el.id)}
								/>
								{el.editable && (
									<Button
										text="edit"
										className="list__edit-btn"
										onClick={this.editHandler.bind(null, el.id)}
									/>
								)}
								<h4 className="list__item-title">{el.firstName}</h4>
								<p className="list__item-age">Age: {el.age}</p>
								<ul className="list__item-ul">
									{el.hobby.map(elem => (
										<li className="list__item-li" key={elem}>
											{elem}
										</li>
									))}
								</ul>
							</div>
						))}
					</div>

					<Form
						rmHobby={this.rmHobby}
						addHobby={this.addHobby}
						editableForm={this.state.editableForm}
						reset={this.resetFormHandler}
						onChangeEvent={this.onChange}
						onSubmitEvent={this.onSubmit}
						values={this.state.item}
					/>
				</div>
			</>
		);
	}
}

export default HOCcomponent(Main);
